'''
Red neuronal profunda para estimar si un paciente tiene o no diabetes
en base a otros parámetros médicos

v3: versión que mejora el "overfitting" con "early stopping" sobre la
    configuración inicial de v1
'''

import pandas as pd
import matplotlib.pyplot as plt
from sklearn.preprocessing import StandardScaler
from keras.models import Sequential
from keras.layers import Dense, Input
from keras.utils import set_random_seed
from keras.callbacks import EarlyStopping
from numpy.random import seed

set_random_seed(0)
seed(0)

# Preparación de datos

datos = pd.read_csv('diabetes.csv')
scaler_X = StandardScaler()
X = datos.iloc[:, :-1]
X = scaler_X.fit_transform(X)
y = datos.iloc[:, -1]

# Definición del modelo

modelo = Sequential([
    Input(shape=(X.shape[1],)),
    Dense(800, activation='relu'),
    Dense(300, activation='relu'),
    Dense(1, activation='sigmoid')
])

modelo.compile(loss='binary_crossentropy', optimizer='sgd', metrics=['accuracy'])
early_stop = EarlyStopping(monitor='val_loss', patience=5, \
    restore_best_weights = True)
datos_entrenamiento = modelo.fit(X, y, validation_split=0.2, epochs=200, \
    callbacks=[early_stop])
    
# Gráficas de costes

plt.plot(datos_entrenamiento.history['loss'])
plt.plot(datos_entrenamiento.history['val_loss'])
plt.title('Evolución del coste')
plt.ylabel('Coste')
plt.xlabel('Epoch')
plt.legend(['Entrenamiento', 'Test'], loc = 'upper left')
plt.show()
